clc;
clear;
clear cfg

% -----------------------------------------------------
%                Create the mesh
% ------------------------------------------------------
[InputGeometry.nodes,InputGeometry.ConnectivityList]=genT6mesh(0:1:20,0:1:20,0:2:20);
% [InputGeometry.nodes,InputGeometry.ConnectivityList]=genT6mesh(0:1:40,0:1:40,0:2:40); % < -------------------- matlab crashes -----------------

% The geometry is filled with random concentration values from 0 to 1. 0:
% 0% tissue 1 100% tissue 2
InputGeometry.concentration = round(rand(size(InputGeometry.nodes(:,1),1),1),4);

%% -----------------------------------------------------
%                 CUSTOMIZATION
% ------------------------------------------------------  


%% ----------------- tissue type 1 ----------------------
mua_t1 = 0.01;      %
musp_t1= 0.08;    % 

g_t1 = 0.85;       % 
n_t1 = 1.33;          % 

% Caclulate mus from musp and g 
mus_t1 = musp_t1/(1-g_t1);

% store in the data struct
InputGeometry.TissueTyp1 = [mua_t1 mus_t1 g_t1 n_t1];


%% ----------------- tissue type 2 ----------------------

% Tissue optical properties of tissue type 2
mua_t2 = 0.07;      % 
musp_t2 = 0.005;    % 

g_t2 = 0.9;       % 
n_t2 = 1.25;          % 

% Caclulate mus from musp and g 
mus_t2 = musp_t2/(1-g_t2);

% store in the data struct
InputGeometry.TissueTyp2 = [mua_t2 mus_t2 g_t2 n_t2];

%% -----------------------------------------------------
%                 OpticalPropterties
% ------------------------------------------------------

dx = max(InputGeometry.nodes(:,1));
dy = max(InputGeometry.nodes(:,2));
dz = max(InputGeometry.nodes(:,3));


% -------- Volume properties   ---------

% Create an ID list for all tetrahedrons: Will be filled within the loop
InputGeometry.ElemTypID = ones(size(InputGeometry.ConnectivityList,1),1);
InputGeometry.TypOpticalProp =  ones(size(InputGeometry.ConnectivityList,1),4);

for k=1:size(InputGeometry.ElemTypID,1) % loops over all tetrahedrons
        
    %get the indax of all nodes corresponding to the k-th tetrahedron
    kElement = InputGeometry.ConnectivityList(k,:);
    
    % get the mean concentration of fibro glandular tissue for 4 nodes
    cNode = 0;
    for kNode = 1 : 4
        cNode = cNode + InputGeometry.concentration(kElement(kNode));
    end
    c_kElement = cNode/4; % Average concentration of the kth element
    
   InputGeometry.ElemTypID(k) = k;
    mua_kElement =  C2OptProp(c_kElement,InputGeometry.TissueTyp1(1),InputGeometry.TissueTyp2(1));
    mus_kElement =  C2OptProp(c_kElement,InputGeometry.TissueTyp1(2),InputGeometry.TissueTyp2(2));
    g_kElement   =  C2OptProp(c_kElement,InputGeometry.TissueTyp1(3),InputGeometry.TissueTyp2(3));
    n_kElement   =  C2OptProp(c_kElement,InputGeometry.TissueTyp1(4),InputGeometry.TissueTyp2(4));
    
    InputGeometry.TypOpticalProp(k,:) = [mua_kElement, mus_kElement, g_kElement, n_kElement];
    
    % Where am I: for big data sets
    if mod(k,round(size(InputGeometry.ElemTypID,1)/100,0)) == 0 % devision without rest: modulus operator
        disp([num2str(round(k/(size(InputGeometry.ElemTypID,1)/100),0)), '% finished']);
    end
    
end

% an N by 4 array, each row specifies [mua, mus, g, n] in order.
% the first row corresponds to medium type 0 which is 
% typically [0 0 1 1]. The second row is type 1, and so on.
% --> Add a row at the beginning
InputGeometry.TypOpticalProp(2:end+1,:) = InputGeometry.TypOpticalProp(1:end,:);
InputGeometry.TypOpticalProp(1,:) = [0, 0, 1, 1];

% get the absorption coefficient for each tetrahedron
ElemAbsorption = InputGeometry.TypOpticalProp(:,2).';

% save('RandomGeometry.mat','InputGeometry') 

%% -----------------------------------------------------
%                 Monte Carlo Simmulation
% ------------------------------------------------------

% assign optical properties
cfg.prop = InputGeometry.TypOpticalProp;

% cfg.prop=[0 0 1 1; 0.01 0.7 0.9 1.33];
cfg.elemprop = InputGeometry.ElemTypID;

% assign xyz of the nodal points
cfg.node = InputGeometry.nodes;

% assign connectivity list of the nodal points (tesselation)
cfg.elem =  InputGeometry.ConnectivityList;

% distance Unit of the mesh
cfg.unitinmm = 1;

% Output Type
cfg.outputtype = 'flux'; 
% cfg.outputtype = 'wl'; % < ------------------------ Matlab crashes ---------------

cfg.nphoton = 1000;  % Total number of photons to be simulated (integer) 1e7
% cfg.nphoton = 1e6; % < ------------------------ Matlab crashes -------------------

% -----------   Isotropic Source   -------------------
cfg.srctype = 'isotropic';      % Source: Point isotropic source
    % 'isotropic' - isotropic source, no param needed
cfg.srcdir=[0 0 1];         % Source direction upwards
cfg.srcpos=[dx/2 dy/2 0];  % Wide-field area source needs to be outside the mesh


% -----------   Time   ----------------------
cfg.tstart=0;
cfg.tend=5e-9; % 1 time stamps
cfg.tstep=5e-9;

% -----------   Detector   ----------------
cfg.detpos= [dx/2 dy/2 dz+1 max(dx,dy)];    % x,y,z, diamter of detector (biggest diameter to cover whole area)
cfg.isreflect = 1;                 % [1] - consider refractive index mismatch: photons esc
cfg.issaveexit = 1;



%% -----------------------------------------------------
%                    RUN Monte Carlo
% ------------------------------------------------------

[flux,detps] = mmclab(cfg);

plotmesh([cfg.node(:,1:3),log10(abs(flux.data))],cfg.elem,'facecolor','interp','linestyle','-','LineWidth',0.01);

%% -----------------------------------------------------
%                 Functions
% ------------------------------------------------------

function OptProp = C2OptProp(c,t1_OptProp,t2_OptProp)
    % According to this formula, the optical properties can be
    % described as the weighted summary of the spatial distribution
    % of tissue 1 and 2
    OptProp = c*t1_OptProp + (1-c)*t2_OptProp;
end

